//
//  SBKRecordFaceView.swift
//  OCR-SDK
//
//  Created by itsol on 8/25/20.
//  Copyright © 2020 itsol. All rights reserved.
//

import UIKit
import AVFoundation
import Vision
import CoreML

open class SBKRecordFaceView: UIView, AVCaptureVideoDataOutputSampleBufferDelegate {
    let nibName = "SBKRecordFaceView"
    var contentView: UIView?
    
    @IBOutlet weak var viewBackground: UIView!
    @IBOutlet weak var lbDescription: UILabel!
    @IBOutlet weak var viewOval: UIView!
    @IBOutlet weak var viewCheckStep1: UIView!
    @IBOutlet weak var viewCheckStep2: UIView!
    @IBOutlet weak var viewCheckStep3: UIView!
    @IBOutlet weak var imageDescription: UIImageView!
    @IBOutlet weak var stackView: UIStackView!
    @IBOutlet weak var stepView: UIView!
    @IBOutlet weak var viewSubIcon: UIView!
    
    private let captureSession = AVCaptureSession()
    private lazy var previewLayer = AVCaptureVideoPreviewLayer(session: self.captureSession)
    private let videoDataOutput = AVCaptureVideoDataOutput()
    
    private var modelDataHandler: SBKModelDataHandler? = SBKModelDataHandler(modelFileInfo: MobileNet.faceModel)
    private var result: [Float]?
    private var previousInferenceTimeMs: TimeInterval = Date.distantPast.timeIntervalSince1970 * 1000
    private let delayBetweenInferencesMs: Double = 1000
    var numberPass = 0
    var numberTrue = 0
    var numberFalse = 0
    var space: Float = 0.0
    var dataImageSuccess: [UIImage] = []
    var checkStartRecord: Bool = false
    var timer = Timer()
    var timeRecord: Int = 0
    var checkStatusRecord: Bool = false
    private var screenHeight: CGFloat = 0
    var idFront: String = ""
    var idBack: String = ""
    var URLToken: String = ""
    var completionSuccessFaceRecord: ([String:Any])->Void = {_ in}
    public var completionSuccessFaceRecordStep: (_ validate: ValidateFace?, _ step: StepRecord?, _ urlImages: [String: Any]?)->Void = {_,_,_ in}
    public var timeSpace: Int = 3
    public var zoom: CGFloat = 1.0
    public var imageStartRecord: UIImage?
    public var checkStep: Int = 0
    private var startusCheck : Bool = false
    public required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        commonInit()
    }
    
    public override init(frame: CGRect) {
        super.init(frame: frame)
        commonInit()
    }
    
    func commonInit() {
        guard let view = loadViewFromNib() else { return }
        view.frame = self.bounds
        self.addSubview(view)
        contentView = view
        if AVCaptureDevice.authorizationStatus(for: .video) ==  .denied {
            return
        }
        self.checkScreen()
        self.loadCamera()
        modelDataHandler?.subtract = 0
    }
    
    func loadViewFromNib() -> UIView? {
        let bundle = Bundle(for: type(of: self))
        let nib = UINib(nibName: nibName, bundle: bundle)
        return nib.instantiate(withOwner: self, options: nil).first as? UIView
    }
    
    func loadCamera() {
        self.addCameraInput()
        self.showCameraFeed()
        self.getCameraFrames()
        
        DispatchQueue.main.async {
            self.captureSession.startRunning()
        }
    }
    
    public func setCameraChecker(){
        self.startusCheck = true
    }
    
    public func statusSubView(statusHide: Bool) {
        viewSubIcon.isHidden = statusHide
    }
    
    public func startCamera() {
        self.captureSession.startRunning()
    }
    
    public func stopCamera() {
        self.captureSession.stopRunning()
    }
    
    public func setOverlay(type: TypeOverlay) {
        
        switch type {
        case .OVAL:
            print("oval")
        case .SQUARE:
            if let viewWithTag = self.viewWithTag(1) {
                viewWithTag.removeFromSuperview()
            }
        case .CIRCLE:
            if let viewWithTag = self.viewWithTag(1) {
                viewWithTag.removeFromSuperview()
            }
            
            let locationTop = self.bounds.origin.y + 35
            
            let originYLBDescription = self.viewSubIcon.frame.origin.y / 896 * self.bounds.height
            
            let heightOval = originYLBDescription - locationTop
            
            let viewOval = UIView(frame: CGRect(x: CGFloat( Int(self.bounds.origin.x + 40 / 896 * self.bounds.height)), y: CGFloat( Int(locationTop)), width: CGFloat( Int(self.bounds.width - 80 / 896 * self.bounds.height)).rounded(.up), height: CGFloat( Int(heightOval - 65))))
            
            viewOval.backgroundColor = UIColor.black.withAlphaComponent(0.6)
            viewOval.tag = 2
            self.addSubview(viewOval)
            
            self.reloadInputViews()
            let pathss: CGMutablePath = {
                let path = CGMutablePath()
                if viewOval.frame.height > viewOval.frame.width {
                    path.addArc(center: CGPoint(x: viewOval.frame.width / 2, y: viewOval.frame.height / 2),
                                radius: viewOval.frame.width / 2,
                                startAngle: 0.0,
                                endAngle: 2.0 * .pi,
                                clockwise: false)
                } else {
                    path.addArc(center: CGPoint(x: viewOval.frame.width / 2, y: viewOval.frame.height / 2),
                                radius: viewOval.frame.height / 2,
                                startAngle: 0.0,
                                endAngle: 2.0 * .pi,
                                clockwise: false)
                }
                return path
            }()
            
            pathss.addRect(CGRect(origin: .zero, size: viewOval.frame.size))
            let maskLayer = CAShapeLayer(layer: viewOval.layer)
            maskLayer.backgroundColor = UIColor.black.cgColor
            maskLayer.path = pathss
            maskLayer.fillRule = .evenOdd
            viewOval.layer.mask = maskLayer
            viewOval.clipsToBounds = true
        }
        
    }
    
    func createOverlay(frame: CGRect, xOffset: CGFloat, yOffset: CGFloat, radius: CGFloat) {
        let locationTop = self.bounds.origin.y + 35
        
        let originYLBDescription = self.viewSubIcon.frame.origin.y / 896 * self.bounds.height
        
        let heightOval = originYLBDescription - locationTop
        
        let viewOval = UIView(frame: CGRect(x: CGFloat( Int(self.bounds.origin.x + 40 / 896 * self.bounds.height)), y: CGFloat( Int(locationTop)), width: CGFloat( Int(self.bounds.width - 80 / 896 * self.bounds.height)).rounded(.up), height: CGFloat( Int(heightOval - 65))))
        
        viewOval.backgroundColor = UIColor.black.withAlphaComponent(0.6)
        viewOval.tag = 1
        self.addSubview(viewOval)
        
        self.reloadInputViews()
        let pathss: CGMutablePath = {
            let path = CGMutablePath()
            path.move(to: CGPoint(x: 0, y: viewOval.frame.height/2))
            if viewOval.frame.width < viewOval.frame.height {
                path.addLine(to: CGPoint(x: 0, y: 0 + viewOval.frame.width / 2))
            }
            path.addQuadCurve(to: CGPoint(x: viewOval.frame.width / 2, y: 0), control: CGPoint(x: 0, y: 0))
            path.addQuadCurve(to: CGPoint(x: viewOval.frame.width, y: 0 + viewOval.frame.width / 2), control: CGPoint(x: viewOval.frame.width, y: 0))
            if viewOval.frame.width < viewOval.frame.height {
                path.addLine(to: CGPoint(x: viewOval.frame.width, y: viewOval.frame.height / 2))
            }
            path.addLine(to: CGPoint(x: viewOval.frame.width, y: viewOval.frame.height - viewOval.frame.width / 2))
            path.addQuadCurve(to: CGPoint(x: viewOval.frame.width / 2, y: viewOval.frame.height), control: CGPoint(x: viewOval.frame.width, y: viewOval.frame.height))
            path.addQuadCurve(to: CGPoint(x: 0, y: viewOval.frame.height -  viewOval.frame.width / 2), control: CGPoint(x: 0, y: viewOval.frame.height))
            path.addLine(to: CGPoint(x: 0, y: viewOval.frame.height / 2))
            return path
        }()
        
        pathss.addRect(CGRect(origin: .zero, size: viewOval.frame.size))
        let maskLayer = CAShapeLayer(layer: viewOval.layer)
        maskLayer.backgroundColor = UIColor.black.cgColor
        maskLayer.path = pathss
        maskLayer.fillRule = .evenOdd
        viewOval.layer.mask = maskLayer
        viewOval.clipsToBounds = true
        
    }
    
    func getMaxResult(result: [Float]) -> Int {
        var max = result[0]
        var index = 0
        for i in 0..<result.count {
            index = max < result[i] ? i : index
            max = max < result[i] ? result[i] : max
        }
        return index
    }
    
    
    func didOutput(pixelBuffer: CVPixelBuffer, statusFace: StatusFace, pureImage: CVPixelBuffer) {
        let currentTimeMs = Date().timeIntervalSince1970 * 1000
        guard (currentTimeMs - previousInferenceTimeMs) >= delayBetweenInferencesMs else { return }
        previousInferenceTimeMs = currentTimeMs
        
        // Pass the pixel buffer to TensorFlow Lite to perform inference.
        result = modelDataHandler?.runModel(onFrame: pixelBuffer)
        if let result = result {
            let maxResult = getMaxResult(result: result)
            if self.checkStatusRecord {
                maxResult == 0 ? (self.numberTrue += 1) : (self.numberFalse += 1)
                self.numberPass += 1
                DispatchQueue.main.async {
                    let ciimage : CIImage = CIImage(cvPixelBuffer: pureImage)
                    let imageView : UIImage = SBKValidateInput.shared.convertCIToUIImage(cmage: ciimage)
                    if maxResult == 0 {
                        if statusFace == .STRAIGHTFACE && self.checkStep == 0 {
                            if self.dataImageSuccess.count == 0 {
                                self.dataImageSuccess.append(imageView)
                            }
                            self.completionSuccessFaceRecordStep(.FACE_STRAIGHT, nil, nil)
                            self.lbDescription.textColor = UIColor.green
                            self.lbDescription.text = "Exactly".localized()
                            self.checkStep = 1
                            self.viewCheckStep1.backgroundColor = UIColor.colorFromHexa("#FBA02E")
                        } else if statusFace != .STRAIGHTFACE && self.checkStep == 0 {
                            self.lbDescription.textColor = UIColor.red
                            self.lbDescription.text = "Please look straight".localized()
                            self.viewCheckStep1.backgroundColor = UIColor.colorFromHexa("#FE3500")
                            self.completionSuccessFaceRecordStep(.FACE_STRAIGHT_FAILED, nil, nil)
                        } else if statusFace == .TORIGHT && self.checkStep == 1 {
                            self.lbDescription.textColor = UIColor.green
                            self.lbDescription.text = "Exactly".localized()
                            self.checkStep = 2
                            self.viewCheckStep2.backgroundColor = UIColor.colorFromHexa("#FBA02E")
                            self.completionSuccessFaceRecordStep(.FACE_RIGHT, nil, nil)
                            if self.dataImageSuccess.count == 1 {
                                self.dataImageSuccess.append(imageView)
                            }
                        } else if statusFace != .TORIGHT && self.checkStep == 1 {
                            self.lbDescription.textColor = UIColor.red
                            self.lbDescription.text = "Please turn to the right".localized()
                            self.completionSuccessFaceRecordStep(.FACE_RIGHT_FAILED, nil, nil)
                            self.viewCheckStep2.backgroundColor = UIColor.colorFromHexa("#FE3500")
                        } else if statusFace == .TOLEFT && self.checkStep == 2 {
                            self.lbDescription.textColor = UIColor.green
                            self.lbDescription.text = "Exactly".localized()
                            self.checkStep = 3
                            self.viewCheckStep3.backgroundColor = UIColor.colorFromHexa("#FBA02E")
                            if self.dataImageSuccess.count == 2 {
                                self.dataImageSuccess.append(imageView)
                            }
                            self.completionSuccessFaceRecordStep(.FACE_LEFT, nil, nil)
                        } else if statusFace != .TOLEFT && self.checkStep == 2 {
                            self.lbDescription.textColor = UIColor.red
                            self.lbDescription.text = "Please turn to the left".localized()
                            self.completionSuccessFaceRecordStep(.FACE_LEFT_FAILED, nil, nil)
                            self.viewCheckStep3.backgroundColor = UIColor.colorFromHexa("#FE3500")
                        }
                    } else {
                        self.lbDescription.textColor = UIColor.red
                        self.lbDescription.text = "Incorrect face, please check!".localized()
                        self.completionSuccessFaceRecordStep(.FACE_FAKE, nil, nil)
                    }
                }
            } else {
                if maxResult == 0 {
                    DispatchQueue.main.async {
                        self.checkStartRecord = true
                        self.lbDescription.textColor = UIColor.white
                        if !self.checkStatusRecord {
                            self.checkStatusRecord = true
                            self.timeRecord = 0
                            self.checkStep = 0
                            self.viewCheckStep1.backgroundColor = .red
                            self.viewCheckStep2.backgroundColor = UIColor.colorFromHexa("#333333")
                            self.viewCheckStep3.backgroundColor = UIColor.colorFromHexa("#333333")
                        }
                        self.completionSuccessFaceRecordStep(.FACE_READY, nil, nil)
                    }
                } else {
                    DispatchQueue.main.async {
                        self.checkStartRecord = false
                        self.lbDescription.textColor = UIColor.red
                        self.lbDescription.text = "Incorrect face, please check!".localized()
                        self.completionSuccessFaceRecordStep(.FACE_FAKE, nil, nil)
                    }
                }
            }
        }
 
    }
    
    @IBAction func onRecord(_ sender: Any) {
        if !self.checkStatusRecord && self.checkStartRecord {
            self.startTimer()
            self.checkStatusRecord = true
            self.timeRecord = 0
            //let image = UIImage(named: "player_stop", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil)
            //self.btnRecord.setImage(imageStartRecord ?? image, for: .normal)
        }
    }
    
    public func captureOutput(_ output: AVCaptureOutput, didOutput sampleBuffer: CMSampleBuffer, from connection: AVCaptureConnection) {
        
        guard let frame = CMSampleBufferGetImageBuffer(sampleBuffer) else {
            debugPrint("unable to get image from sample buffer")
            return
        }
        let pureImage = self.resizeImageFace(pixelBuffer: frame)
        if self.startusCheck {
            if #available(iOS 11.0, *)   {
                if self.screenHeight == 2436 {
                 
                    self.detectFace(in: self.resizeImageFace(pixelBuffer: frame.resized(to: CGSize(width: 360, height: 480))!), pureImage: pureImage)
                } else {
                    self.detectFace(in: pureImage, pureImage: pureImage)
                }
                
            }
        }
    }
    
    func checkScreen(){
        if UIDevice().userInterfaceIdiom == .phone {
            self.screenHeight = UIScreen.main.nativeBounds.height
        }
    }
    
    private func addCameraInput() {
        if self.screenHeight == 2436 {
            self.captureSession.sessionPreset = .high
        } else {
            self.captureSession.sessionPreset = .medium
        }
        if #available(iOS 11.1, *) {
            guard let device = AVCaptureDevice.DiscoverySession(
                deviceTypes: [.builtInWideAngleCamera, .builtInDualCamera, .builtInTrueDepthCamera],
                mediaType: .video,
                position: .front).devices.first else {
                    print("No back camera device found, please make sure to run SimpleLaneDetection in an iOS device and not a simulator")
                    return
                }
            let cameraInput = try! AVCaptureDeviceInput(device: device)
            if captureSession.canAddInput(cameraInput) {
                self.captureSession.addInput(cameraInput)
            }
            
            do {
                try device.lockForConfiguration()
                defer { device.unlockForConfiguration() }
                device.videoZoomFactor = self.zoom
            } catch {
                print("\(error.localizedDescription)")
            }
            
        }
    }
    
    private func showCameraFeed() {
        self.previewLayer.videoGravity = .resizeAspectFill
        self.viewBackground.layer.addSublayer(self.previewLayer)
        self.previewLayer.frame = self.bounds
        
        let overlayOval = createOverlayOval(frame: self.bounds, xOffset: viewBackground.frame.midX, yOffset: viewBackground.frame.midY, radius: 50.0)
        self.viewBackground.addSubview(overlayOval)
        
        self.createOverlay(frame: self.bounds, xOffset: self.bounds.midX, yOffset: self.bounds.midY, radius: 50.0)
    }
    
    func createOverlayOval(frame: CGRect, xOffset: CGFloat, yOffset: CGFloat, radius: CGFloat) -> UIView {
        let overlayView = UIView(frame: frame)
        overlayView.backgroundColor = UIColor.black.withAlphaComponent(0.6)
        let path = CGMutablePath()
        
        let locationTop = self.bounds.origin.y + 35
        
        let originYLBDescription = self.viewSubIcon.frame.origin.y / 896 * self.bounds.height
        
        let heightOval = originYLBDescription - locationTop
        
        path.addRoundedRect(in: CGRect(x: CGFloat( Int(self.bounds.origin.x + 40 / 896 * self.bounds.height)), y: CGFloat( Int(locationTop)), width: CGFloat( Int(self.bounds.width - 80 / 896 * self.bounds.height)), height: CGFloat( Int(heightOval - 65))), cornerWidth: 0, cornerHeight: 0)
        
        path.addRect(CGRect(origin: .zero, size: overlayView.frame.size))
        let maskLayer = CAShapeLayer()
        maskLayer.backgroundColor = UIColor.black.cgColor
        maskLayer.path = path
        maskLayer.fillRule = .evenOdd
        overlayView.layer.mask = maskLayer
        overlayView.clipsToBounds = true
        
        return overlayView
    }
    
    private func getCameraFrames() {
        self.videoDataOutput.videoSettings = [(kCVPixelBufferPixelFormatTypeKey as NSString) : NSNumber(value: kCVPixelFormatType_32BGRA)] as [String : Any]
        self.videoDataOutput.alwaysDiscardsLateVideoFrames = true
        self.videoDataOutput.setSampleBufferDelegate(self, queue: DispatchQueue(label: "camera_frame_processing_queue"))
        self.captureSession.addOutput(self.videoDataOutput)
        guard let connection = self.videoDataOutput.connection(with: AVMediaType.video), connection.isVideoOrientationSupported else { return }
        connection.videoOrientation = .portrait
    }
    
    @available(iOS 11.0, *)
    private func detectFace(in image: CVPixelBuffer, pureImage: CVPixelBuffer) {
        let faceDetectionRequest = VNDetectFaceLandmarksRequest(completionHandler: { (request: VNRequest, error: Error?) in
            DispatchQueue.main.async {
                DispatchQueue.main.asyncAfter(deadline: .now() + 1.0) {
                    if (request.results as? [VNFaceObservation]) != nil {
                        if let results = request.results as? [VNFaceObservation], results.count > 0 {
                            
                            if results.count > 1 {
                                self.lbDescription.textColor = UIColor.red
                                self.lbDescription.text = "There are many faces in the frame".localized()
                                self.completionSuccessFaceRecordStep(.MANY_FACE, nil, nil)
                            } else {
                                let statusString = self.checkFaceRightLeft(landmarks: results[0].landmarks!)
                                
                                self.startTimer()
                                
                                DispatchQueue.global().async {
                                    self.didOutput(pixelBuffer: image, statusFace: statusString, pureImage: pureImage)
                                }
                            }
                        } else {
                            if !self.checkStatusRecord {
                                self.checkStartRecord = false
                                self.lbDescription.textColor = UIColor.red
                                self.lbDescription.text = "Position your face in the oval".localized()//"Đặt vị trí mặt bạn vào hình"
                                self.completionSuccessFaceRecordStep(.NO_FACE, nil, nil)
                            } else {
                                self.lbDescription.textColor = UIColor.red
                                self.lbDescription.text = "Incorrect face, please check!".localized()
                                
                                DispatchQueue.global().async {
                                    self.didOutput(pixelBuffer: image, statusFace: .ERROR, pureImage: pureImage)
                                }
                            }
                        }
                    }
                }
            }
        })
        
        if #available(iOS 12.0, *) {
            // Force the revision to 2 (68-points) even on iOS 13 or greater
            // when VNDetectFaceLandmarksRequestRevision3 is available.
            faceDetectionRequest.revision = 2
        }
        
        let imageRequestHandler = VNImageRequestHandler(cvPixelBuffer: image, orientation: .leftMirrored, options: [:])
        try? imageRequestHandler.perform([faceDetectionRequest])
    }
    
    func resetRecord() {
        self.numberPass = 0
        self.numberTrue = 0
        self.numberFalse = 0
        self.space = 0.0
        self.dataImageSuccess.removeAll()
        self.checkStartRecord = false
        self.checkStatusRecord = false
        
        //let image = UIImage(named: "ic_record", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil)
        //self.btnRecord.setImage(image, for: .normal)
    }
    
    @objc func setLableRecord(timer: Timer) {
        self.timeRecord += 1
        self.completionSuccessFaceRecordStep(nil, nil, nil)
        switch self.timeRecord {
        case 1:
            self.lbDescription.textColor = UIColor.white
            self.lbDescription.text = " ".localized()//"vui lòng nhìn thẳng"
            self.completionSuccessFaceRecordStep(nil, .STEP_FACE_STRAIGHT, nil)
        case self.timeSpace:
            self.lbDescription.textColor = UIColor.white
            self.lbDescription.text = " ".localized()//"Quay sang phải"
            self.completionSuccessFaceRecordStep(nil, .STEP_FACE_RIGHT, nil)
        case self.timeSpace * 2:
            self.lbDescription.textColor = UIColor.white
            self.lbDescription.text = " ".localized()//"Quay sang trái"
            self.completionSuccessFaceRecordStep(nil, .STEP_FACE_LEFT, nil)
        case self.timeSpace * 3:
            let pass: Float = Float(self.numberTrue)/Float(self.numberPass)
            if Global.ratioPass < pass * 100 && self.dataImageSuccess.count > 2 {
                DispatchQueue.main.async {
                    guard let urlImageStraight = SBKValidateInput.shared.saveImage(imageName: "imageStraightFile.png", image: self.dataImageSuccess[0]),
                          let urlImageRight = SBKValidateInput.shared.saveImage(imageName: "imageRightFile.png", image: self.dataImageSuccess[1]),
                          let urlImageLeft = SBKValidateInput.shared.saveImage(imageName: "imageLeftFile.png", image: self.dataImageSuccess[2]) else {
                              return
                          }
                    let data: [String: Any] = [
                        "imageStraightFile": urlImageStraight,
                        "imageRightFile": urlImageRight,
                        "imageLeftFile": urlImageLeft,
                    ]
                    self.completionSuccessFaceRecordStep(nil, nil, data)
                    self.timeRecord = 0
                    self.resetRecord()
                    self.timer.invalidate()
                }
            } else {
                self.lbDescription.textColor = UIColor.red
                self.lbDescription.text = " ".localized()//"Chưa đạt yêu cầu, hãy thử lại!"
                self.completionSuccessFaceRecordStep(.INVALID, nil, nil)
            }
        case self.timeSpace * 4:
            self.timer.invalidate()
            self.resetRecord()
            self.timeRecord = 0
        default:
            break
        }
        
    }
    
    func startTimer() {
        
        switch self.checkStep {
        case 0:
            self.imageDescription.image = UIImage(named: "Scan-3@2x", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil)
            self.lbDescription.textColor = UIColor.white
            self.completionSuccessFaceRecordStep(nil, .STEP_FACE_STRAIGHT, nil)
            self.lbDescription.text = "Please look straight".localized()//"vui lòng nhìn thẳng"
        case 1:
            self.imageDescription.image = UIImage(named: "Scan-5@2x", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil)
            self.lbDescription.textColor = UIColor.white
            self.completionSuccessFaceRecordStep(nil, .STEP_FACE_RIGHT, nil)
            self.lbDescription.text = "Please turn to the right".localized()//"Quay sang phải"
        case 2:
            self.imageDescription.image = UIImage(named: "Scan-4@2x", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil)
            self.lbDescription.textColor = UIColor.white
            self.completionSuccessFaceRecordStep(nil, .STEP_FACE_LEFT, nil)
            self.lbDescription.text = "Please turn to the left".localized()//"Quay sang trái"
        case 3:
            if self.dataImageSuccess.count > 2 {
                DispatchQueue.main.async {
                    guard let urlImageStraight = SBKValidateInput.shared.saveImage(imageName: "imageStraightFile.png", image: self.dataImageSuccess[0]),
                          let urlImageRight = SBKValidateInput.shared.saveImage(imageName: "imageRightFile.png", image: self.dataImageSuccess[1]),
                          let urlImageLeft = SBKValidateInput.shared.saveImage(imageName: "imageLeftFile.png", image: self.dataImageSuccess[2]) else {
                              return
                          }
                    let data: [String: Any] = [
                        "imageStraightFile": urlImageStraight,
                        "imageRightFile": urlImageRight,
                        "imageLeftFile": urlImageLeft,
                    ]
                    self.completionSuccessFaceRecordStep(nil, nil, data)
                    self.timeRecord = 0
                    self.resetRecord()
                    self.timer.invalidate()
                }
            } else {
                self.lbDescription.textColor = UIColor.red
                self.lbDescription.text = "Unsatisfactory, please try again!".localized()//"Chưa đạt yêu cầu, hãy thử lại!"
                self.completionSuccessFaceRecordStep(.INVALID, nil, nil)
            }
        default:
            break
        }
    }
    
    func resizeImageFace(pixelBuffer: CVPixelBuffer) -> CVPixelBuffer {
        let validateView = SBKValidateInput.shared
        let imageInput = validateView.convertCVPixelToUIImage(pixelBuffer: pixelBuffer)
        let imageOutput = validateView.cropImageFace(image: imageInput, rect: CGRect(x: imageInput.size.width / 10, y: imageInput.size.height / 14, width: imageInput.size.width * 8 / 10, height: imageInput.size.height * 9 / 14 ), scale: 1.0)
        
        let ciimage = CIImage(image: imageOutput!)
        let eaglContext = EAGLContext(api: .openGLES2)
        let tmpcontext = CIContext(eaglContext: eaglContext!)
        let cgimage =  tmpcontext.createCGImage(ciimage!, from: ciimage!.extent)
        
        return validateView.convertCGImgeToCVPixelBuffer(forImage: cgimage!)!
    }
    
    @available(iOS 11.0, *)
    func checkFaceRightLeft(landmarks: VNFaceLandmarks2D) -> StatusFace {
        let face = landmarks.faceContour?.normalizedPoints
        let nose = landmarks.nose?.normalizedPoints
        
        let faceLeftPoint = face![0]
        let faceRightPoint = face![face!.count - 1]
        let faceBottomPoint = face![5]
        let nosePoint = nose![4]
        
        let sRight = (faceBottomPoint.x - faceLeftPoint.x) * (nosePoint.y - faceLeftPoint.y) - (nosePoint.x - faceLeftPoint.x) * (faceBottomPoint.y - faceLeftPoint.y)
        let sLeft = (faceBottomPoint.x - faceRightPoint.x) * (nosePoint.y - faceRightPoint.y) - (nosePoint.x - faceRightPoint.x) * (faceBottomPoint.y - faceRightPoint.y)
        
        let sFaceRight = sqrt(sRight * sRight) / 2
        let sFaceLeft = sqrt(sLeft * sLeft) / 2
        let totalS = sFaceLeft + sFaceRight
        
        if sFaceLeft / totalS > 0.6 {
            return .TOLEFT
        } else if sFaceRight / totalS > 0.6 {
            return .TORIGHT
        } else {
            return .STRAIGHTFACE
        }
    }
}
