//
//  ExtUIImage.swift
//  OCR-SDK
//
//  Created by Nguyen Van An on 4/26/21.
//  Copyright © 2021 itsol. All rights reserved.
//

import Foundation
import UIKit

extension UIImage {
    //Xử lý ảnh hiển thị
    func crop(rect: CGRect, scale: CGFloat) -> UIImage? {
        UIGraphicsBeginImageContextWithOptions(CGSize(width: rect.width, height: rect.height), true, 0.0)
        self.draw(at: CGPoint(x: -rect.minX / scale, y: -rect.minY / scale))
        let croppedImage = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
        return croppedImage
    }
    
    
    func getPixelColor(pos: CGPoint, dataImage: Data, image: UIImage) -> UIColor {
        let pixelData = image.cgImage!.dataProvider!.data
        let data: UnsafePointer<UInt8> = CFDataGetBytePtr(pixelData)
        let pixelInfo: Int = ((Int(image.size.width) * Int(pos.y)) + Int(pos.x)) * 4
        let r = CGFloat(dataImage[pixelInfo]) / CGFloat(255.0)
        let g = CGFloat(dataImage[pixelInfo+1]) / CGFloat(255.0)
        let b = CGFloat(dataImage[pixelInfo+2]) / CGFloat(255.0)
        let a = CGFloat(dataImage[pixelInfo+3]) / CGFloat(255.0)

        return UIColor(red: r, green: g, blue: b, alpha: a)
    }
    
    func rotate(radians: Float) -> UIImage? {
        var newSize = CGRect(origin: CGPoint.zero, size: self.size).applying(CGAffineTransform(rotationAngle: CGFloat(radians))).size
        newSize.width = floor(newSize.width)
        newSize.height = floor(newSize.height)
        UIGraphicsBeginImageContextWithOptions(newSize, false, self.scale)
        let context = UIGraphicsGetCurrentContext()!
        context.translateBy(x: newSize.width/2, y: newSize.height/2)
        context.rotate(by: CGFloat(radians))
        self.draw(in: CGRect(x: -self.size.width/2, y: -self.size.height/2, width: self.size.width, height: self.size.height))
        let newImage = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
        return newImage
    }
    
    func resize(targetSize: CGSize) -> UIImage {
        let size = self.size
        let widthRatio  = targetSize.width  / self.size.width
        let heightRatio = targetSize.height / self.size.height
        var newSize: CGSize
        if(widthRatio > heightRatio) {
            newSize = CGSize(width: size.width * heightRatio, height: size.height * heightRatio)
        } else {
            newSize = CGSize(width: size.width * widthRatio,  height: size.height * widthRatio)
        }
        let rect = CGRect(x: 0, y: 0, width: newSize.width, height: newSize.height)
        UIGraphicsBeginImageContextWithOptions(newSize, false, 1.0)
        self.draw(in: rect)
        let newImage = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
        return newImage!
    }
    
    func toCVPixel() -> CVPixelBuffer {
        let ciimage = CIImage(image: self)
        let eaglContext = EAGLContext(api: .openGLES2)
        let tmpcontext = CIContext(eaglContext: eaglContext!)
        let cgimage = tmpcontext.createCGImage(ciimage!, from: ciimage!.extent)
        return cgimage!.toCVPixelBuffer()!
    }
    
}
